@@ -108,12 +108,20 @@ async def decode_token(auth: str):
 async def get_user(
     payload: dict, token: str, get_profile: bool = True
 ) -> Optional[AuthUser]:
+    settings = get_settings()
     user: Optional[AuthUser] = None
     token_scopes: List[str] = []
+    roles: List[str] = []
 
     if "permissions" in payload and payload.get("permissions") is not None:
         token_scopes = payload["permissions"]
 
+    if (
+        settings.ROLES_NAMESPACE in payload
+        and payload.get(settings.ROLES_NAMESPACE) is not None
+    ):
+        roles = payload[settings.ROLES_NAMESPACE]
+
     if "sub" in payload:
         if (
             get_profile
@@ -135,16 +143,23 @@ async def get_user(
         if len(token_scopes) > 0:
             user.scopes = token_scopes
 
+        if len(roles) > 0:
+            user.roles = roles
+
     return user
 
 
 async def get_current_user(
-    security_scopes: SecurityScopes, token: str = Depends(oauth2_scheme)
+    security_scopes: SecurityScopes,
+    token: str = Depends(oauth2_scheme),
+    get_profile: bool = None,
 ) -> AuthUser:
     settings = get_settings()
+    if get_profile is None:
+        get_profile = settings.USE_AUTH0_PROFILE
     try:
         payload = await decode_token(token)
-        user = await get_user(payload, token, settings.USE_AUTH0_PROFILE)
+        user = await get_user(payload, token, get_profile)
 
     except (JWTError, ValidationError) as e:
         LOG.error("Could not validate credentials. {}", str(e))
@@ -153,27 +168,14 @@ async def get_current_user(
     if not user:
         raise UnauthorizedError(detail="Invalid authentication credentials.",)
 
-    for scope in security_scopes.scopes:
-        if scope not in user.scopes:
-            raise ForbiddenError(detail="Not enough permissions",)
+    if settings.ENABLE_SCOPES:
+        for scope in security_scopes.scopes:
+            if scope not in user.scopes and scope not in user.roles:
+                raise ForbiddenError(detail="Not enough permissions",)
     return user
 
 
 async def get_auth_simple(
     security_scopes: SecurityScopes, token: str = Depends(oauth2_scheme)
 ) -> AuthUser:
-    try:
-        payload = await decode_token(token)
-        user = await get_user(payload, token, False)
-
-    except (JWTError, ValidationError) as e:
-        LOG.error("Could not validate credentials. {}", str(e))
-        raise UnauthorizedError(detail="Could not validate credentials.",)
-
-    if not user:
-        raise UnauthorizedError(detail="Invalid authentication credentials.",)
-
-    for scope in security_scopes.scopes:
-        if scope not in user.scopes:
-            raise ForbiddenError(detail="Not enough permissions",)
-    return user
+    return await get_current_user(security_scopes, token, False)
