@@ -23,7 +23,7 @@ class DatabaseType(str, Enum):
 
 
 class Settings(BaseSettings):
-    VERSION: str = "0.10.2"
+    VERSION: str = "0.12.1"
     API_VERSION: int = 1
     API_PRE_PATH: str = f"/api/v{API_VERSION}"
 
@@ -56,6 +56,8 @@ class Settings(BaseSettings):
     USE_AUTH0_PROFILE: bool = True
     OAUTH2_AUTH_URL: str = f"https://{AUTH0_DOMAIN}/authorize?audience={API_AUDIENCE}"
     JWT_ALGORITHMS: List[str] = ["RS256"]
+    ROLES_NAMESPACE: str = "http://namespace/roles"
+    ENABLE_SCOPES: bool = True
 
     # logging configuration
     LOGGING_LEVEL: int = logging.DEBUG if DEBUG else logging.INFO
