@@ -3,7 +3,6 @@ from io import BytesIO
 from typing import List, Optional, Union
 
 from fastapi import UploadFile
-from fa_common import FileDownloadRef
 
 from .model import File
 
@@ -93,3 +92,7 @@ class BaseClient(abc.ABC):
         """Enable file to be downloaded without auth via a URL temporarily
         """
         pass
+
+    @abc.abstractclassmethod
+    def get_uri(cls, bucket_name: str, path: str) -> str:
+        pass
