@@ -10,7 +10,7 @@ from minio.error import (
     ResponseError,
 )
 
-from fa_common import StorageError, force_async, get_current_app, FileDownloadRef
+from fa_common import StorageError, force_async, get_current_app
 from fa_common import logger as LOG
 from fa_common import sizeof_fmt
 
@@ -56,6 +56,11 @@ class MinioClient(BaseClient):
     async def bucket_exists(self, name: str) -> bool:
         return await force_async(self.minio.bucket_exists)(name)
 
+    @classmethod
+    def get_uri(cls, bucket_name: str, path: str) -> str:
+        # FIXME: this probably won't work as indented
+        return f"{bucket_name}/{path}"
+
     @classmethod
     def object_to_file(
         cls, obj: Object, bucket_name: str, file_name: str = None
