@@ -2,16 +2,21 @@ import time
 from collections.abc import Iterable
 
 from gitlab import Gitlab
-from gitlab.v4.objects import Project, ProjectPipeline, ProjectPipelineJob, ProjectJob
-from gitlab.exceptions import GitlabGetError, GitlabCreateError, GitlabDeleteError
+from gitlab.v4.objects import Project, ProjectPipeline, ProjectJob
+from gitlab.exceptions import (
+    GitlabGetError,
+    GitlabCreateError,
+    GitlabDeleteError,
+    GitlabJobRetryError,
+)
 
 import yaml
 import json
-from typing import List, Union
+from typing import List, Union, Dict, Optional
 from fa_common import force_async, get_current_app, get_settings
 from fa_common import logger as LOG
 from .utils import GitlabCIYAMLDumper
-from .models import WorkflowRun, JobRun
+from .models import WorkflowRun, JobRun, JobAction
 
 
 class GitlabClient:
@@ -68,7 +73,7 @@ class GitlabClient:
                 {
                     "name": project_name,
                     "namespace_id": settings.GITLAB_GROUP_ID,
-                    "auto_cancel_pending_pipelines": False,
+                    "auto_cancel_pending_pipelines": "disabled",
                 }
             )
         except GitlabCreateError as err:
@@ -115,7 +120,11 @@ class GitlabClient:
             raise err
 
     async def get_job(
-        self, project_id: Union[str, int], job_id: int, include_log: bool = False
+        self,
+        project_id: Union[str, int],
+        job_id: int,
+        include_log: bool = False,
+        include_output=False,
     ) -> JobRun:
         project = await self._get_project(project_id)
         try:
@@ -128,7 +137,7 @@ class GitlabClient:
                 f"No job found with the id {job_id} in project {project_id}"
             )
 
-        return self.job_to_job_run(job, log)
+        return self.job_to_job_run(job, log, include_output)
 
     async def get_job_log(self, project_id: Union[str, int], job_id: int) -> bytes:
         project = await self._get_project(project_id)
@@ -168,27 +177,36 @@ class GitlabClient:
         branch: str,
         variables: List[dict] = None,
         wait: bool = False,
+        include_output: bool = False,
     ) -> WorkflowRun:
         project = await self._get_project(project_id)
         pipeline = project.pipelines.create({"ref": branch, "variables": variables})
         if wait:
             while pipeline.finished_at is None:
+                LOG.info(
+                    f"Waiting for workflow {pipeline.id} to finish, sleeping for 5 seconds"
+                )
+                time.sleep(5)
                 pipeline.refresh()
-                time.sleep(2)
-        jobs = pipeline.jobs.list()
 
-        return await self.pipeline_to_workflow_run(pipeline, project, jobs)
+        return await self.pipeline_to_workflow_run(pipeline, project, include_output)
 
-    def job_to_job_run(self, job: ProjectJob, job_log: bytes = None) -> JobRun:
+    def job_to_job_run(
+        self, job: ProjectJob, job_log: bytes = None, include_output: bool = True,
+    ) -> JobRun:
         output = None
-        if hasattr(job, "artifacts_file") and job.artifacts_file is not None:
+        if (
+            include_output
+            and hasattr(job, "artifacts_file")
+            and job.artifacts_file is not None
+        ):
             try:
                 output_bytes = None
-                output_bytes = job.artifact("output/outputs.json")
+                output_bytes = job.artifact(f"output/{job.id}/outputs.json")
                 if output_bytes is not None:
                     output = json.loads(output_bytes)
                     if isinstance(output, Iterable):
-                        output = List(output)
+                        output = list(output)
             except Exception as err:
                 LOG.error(err)
 
@@ -210,14 +228,25 @@ class GitlabClient:
         return job_run
 
     async def pipeline_to_workflow_run(
-        self,
-        pipeline: ProjectPipeline,
-        project: Project,
-        jobs: List[ProjectPipelineJob] = [],
+        self, pipeline: ProjectPipeline, project: Project, include_output: bool = False,
     ) -> WorkflowRun:
-        job_runs: List[JobRun] = []
+        hidden_jobs: List[JobRun] = []
+        jobs = await force_async(pipeline.jobs.list)()
+        job_dict: Dict[str, JobRun] = {}
         for job in jobs:
-            job_runs.append(await self.get_job(project.id, job.id))
+            project_job = project.jobs.get(job.id)
+            job_run = self.job_to_job_run(project_job, include_output=include_output)
+            if job_dict.get(job_run.name) is not None:
+                if (
+                    job_dict[job_run.name].get_compare_time()
+                    < job_run.get_compare_time()
+                ):
+                    hidden_jobs.append(job_dict[job_run.name])
+                    job_dict[job_run.name] = job_run
+                else:
+                    hidden_jobs.append(job_run)
+            else:
+                job_dict[job_run.name] = job_run
 
         return WorkflowRun(
             id=pipeline.id,
@@ -228,7 +257,8 @@ class GitlabClient:
             started_at=pipeline.started_at,
             finished_at=pipeline.finished_at,
             duration=pipeline.duration,
-            jobs=job_runs,
+            jobs=list(job_dict.values()),
+            hidden_jobs=hidden_jobs,
         )
 
     async def delete_project(self, project_id: int, wait: bool = True):
@@ -258,3 +288,77 @@ class GitlabClient:
                 await self.delete_project(proj.id, wait)
                 return
         raise ValueError(f"No project found with the name {project_name}")
+
+    async def job_action(
+        self, project_id: Union[str, int], job_id: int, action: JobAction
+    ) -> Optional[JobRun]:
+        project = await self._get_project(project_id)
+        try:
+            job = await force_async(project.jobs.get)(job_id)
+            if action is JobAction.PLAY:
+                await force_async(job.play)()
+            elif action is JobAction.CANCEL:
+                await force_async(job.cancel)()
+            elif action is JobAction.RETRY:
+                await force_async(job.retry)()
+                pipeline = await force_async(project.pipelines.get)(job.pipeline["id"])
+                jobs = await force_async(pipeline.jobs.list)()
+                for _job in jobs:
+                    if (
+                        _job.name == job.name
+                        and _job.status in ["pending", "running"]
+                        and _job.id != job.id
+                    ):
+                        job = _job
+                        break
+
+            elif action is JobAction.DELETE:
+                await force_async(job.delete)()
+                return None
+            job = await force_async(project.jobs.get)(job.id)
+            return self.job_to_job_run(job, include_output=False)
+        except GitlabGetError:
+            raise ValueError(
+                f"No job found with the id {job_id} in project {project_id}"
+            )
+        except GitlabJobRetryError:
+            raise ValueError(f"Job {job_id} is not currently retryable.")
+
+    async def get_pipeline(
+        self, project_id: Union[str, int], pipeline_id: int, include_output: bool
+    ) -> WorkflowRun:
+        project = await self._get_project(project_id)
+        try:
+            pipeline = await force_async(project.pipelines.get)(pipeline_id)
+            return await self.pipeline_to_workflow_run(
+                pipeline, project, include_output
+            )
+        except GitlabGetError:
+            raise ValueError(
+                f"No pipeleine found with the id {pipeline_id} in project {project_id}"
+            )
+
+    async def retry_pipeline(
+        self, project_id: Union[str, int], pipeline_id: int
+    ) -> WorkflowRun:
+        project = await self._get_project(project_id)
+        try:
+            pipeline = await force_async(project.pipelines.get)(pipeline_id)
+            await force_async(pipeline.retry)()
+            return await self.pipeline_to_workflow_run(pipeline, project, False)
+        except GitlabGetError:
+            raise ValueError(
+                f"No pipeleine found with the id {pipeline_id} in project {project_id}"
+            )
+
+    async def delete_pipeline(self, project_id: Union[str, int], pipeline_id: int):
+        project = await self._get_project(project_id)
+        try:
+            pipeline = await force_async(project.pipelines.get)(pipeline_id)
+            if pipeline.status in ["running", "pending"]:
+                await force_async(pipeline.cancel)()
+            await force_async(pipeline.delete)()
+        except GitlabGetError:
+            raise ValueError(
+                f"No pipeline found with the id {pipeline_id} in project {project_id}"
+            )
