@@ -74,6 +74,7 @@ async def retry_workflow(user_id: str, workflow_id: int) -> WorkflowRun:
 async def get_job_output(
     bucket_id: str, workflow_id: int, job_id: int
 ) -> Union[dict, List, None]:
+    client = get_workflow_client()
     storage = get_storage_client()
     file = await storage.get_file(
         bucket_id,
@@ -218,7 +219,7 @@ async def get_workflow(
     file_refs: bool = True,
 ) -> WorkflowRun:
     client = get_workflow_client()
-    workflow = await client.get_pipeline(user_id, workflow_id, False)
+    workflow = await client.get_pipeline(user_id, workflow_id, output)
     for job in workflow.jobs:
         job = await add_data_to_job(job, bucket_id, output, file_refs)
     return workflow
